@@ -11,15 +11,6 @@ use std::{
     time::{Duration, Instant},
 };
 
-use frep_core::{
-    line_reader::{BufReadExt, LineEnding},
-    replace::{add_replacement, replacement_if_match},
-    search::{FileSearcher, ParsedSearchConfig, SearchResult, SearchResultWithReplacement},
-    validation::{
-        DirConfig, SearchConfig, ValidationErrorHandler, ValidationResult,
-        validate_search_configuration,
-    },
-};
 use ignore::WalkState;
 use log::{debug, warn};
 use tokio::{
@@ -36,9 +27,16 @@ use crate::{
     errors::AppError,
     fields::{FieldName, SearchFieldValues, SearchFields},
     keyboard::{KeyCode, KeyEvent, KeyModifiers},
+    line_reader::{BufReadExt, LineEnding},
     replace::{self, PerformingReplacementState, ReplaceState},
+    replace::{add_replacement, replacement_if_match},
     search::Searcher,
+    search::{FileSearcher, ParsedSearchConfig, SearchResult, SearchResultWithReplacement},
     utils::{Either, Either::Left, Either::Right, ceil_div},
+    validation::{
+        DirConfig, SearchConfig, ValidationErrorHandler, ValidationResult,
+        validate_search_configuration,
+    },
 };
 
 #[derive(Debug, Clone)]
@@ -89,6 +87,7 @@ pub enum BackgroundProcessingEvent {
 #[derive(Debug)]
 pub enum AppEvent {
     PerformSearch,
+    DismissToast { generation: u64 },
 }
 
 #[derive(Debug)]
@@ -440,10 +439,17 @@ pub enum Popup {
     Text { title: String, body: String },
 }
 
+#[derive(Debug, Clone)]
+struct Toast {
+    message: String,
+    generation: u64,
+}
+
 #[derive(Clone, Debug, PartialEq, Eq)]
 #[allow(clippy::struct_excessive_bools)]
 pub struct AppRunConfig {
     pub include_hidden: bool,
+    pub include_git_folders: bool,
     pub advanced_regex: bool,
     pub immediate_search: bool,
     pub immediate_replace: bool,
@@ -456,6 +462,7 @@ impl Default for AppRunConfig {
     fn default() -> Self {
         Self {
             include_hidden: false,
+            include_git_folders: false,
             advanced_regex: false,
             immediate_search: false,
             immediate_replace: false,
@@ -466,23 +473,69 @@ impl Default for AppRunConfig {
 }
 
 #[derive(Debug)]
-#[allow(clippy::struct_excessive_bools)]
+pub struct EventChannels {
+    pub sender: UnboundedSender<Event>,
+    receiver: UnboundedReceiver<Event>,
+}
+
+impl EventChannels {
+    pub fn new() -> Self {
+        let (sender, receiver) = mpsc::unbounded_channel();
+        Self { sender, receiver }
+    }
+
+    pub async fn recv(&mut self) -> Option<Event> {
+        self.receiver.recv().await
+    }
+}
+
+impl Default for EventChannels {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
+#[derive(Debug)]
+pub struct UIState {
+    pub current_screen: Screen,
+    pub popup: Option<Popup>,
+    toast: Option<Toast>,
+    errors: Vec<AppError>,
+}
+
+impl UIState {
+    pub fn new(current_screen: Screen) -> Self {
+        Self {
+            current_screen,
+            popup: None,
+            toast: None,
+            errors: Vec::new(),
+        }
+    }
+
+    pub fn add_error(&mut self, error: AppError) {
+        self.errors.push(error);
+    }
+
+    pub fn errors(&self) -> &[AppError] {
+        &self.errors
+    }
+
+    pub fn clear_errors(&mut self) {
+        self.errors.clear();
+    }
+}
+
+#[derive(Debug)]
 pub struct App {
     pub config: Config,
     key_map: KeyMap,
-    pub current_screen: Screen,
     pub search_fields: SearchFields,
     pub searcher: Option<Searcher>,
     pub input_source: InputSource,
-    pub event_sender: UnboundedSender<Event>,
-    event_receiver: UnboundedReceiver<Event>,
-    errors: Vec<AppError>,
-    include_hidden: bool,
-    immediate_replace: bool,
-    pub print_results: bool,
-    pub print_on_exit: bool,
-    popup: Option<Popup>,
-    advanced_regex: bool,
+    pub run_config: AppRunConfig,
+    pub event_channels: EventChannels,
+    pub ui_state: UIState,
 }
 
 #[derive(Debug)]
@@ -518,7 +571,7 @@ fn generate_escape_deprecation_message(quit_keymap: Option<KeyEvent>) -> String
 // methods can't express split borrows but macros can
 macro_rules! get_bg_receiver {
     ($self:expr) => {
-        match &mut $self.current_screen {
+        match &mut $self.ui_state.current_screen {
             Screen::SearchFields(SearchFieldsState { search_state, .. }) => {
                 search_state.as_mut().map(|s| &mut s.processing_receiver)
             }
@@ -546,11 +599,9 @@ impl<'a> App {
     pub fn new(
         input_source: InputSource,
         search_field_values: &SearchFieldValues<'a>,
-        app_run_config: &AppRunConfig,
+        app_run_config: AppRunConfig,
         config: Config,
     ) -> anyhow::Result<Self> {
-        let (event_sender, event_receiver) = mpsc::unbounded_channel();
-
         let search_fields = SearchFields::with_values(
             search_field_values,
             config.search.disable_prepopulated_fields,
@@ -563,26 +614,22 @@ impl<'a> App {
 
         let key_map = KeyMap::from_config(&config.keys).map_err(display_conflict_errors)?;
 
+        let search_immediately =
+            app_run_config.immediate_search || !search_field_values.search.value.is_empty();
+
         let mut app = Self {
             config,
             key_map,
-            current_screen: Screen::SearchFields(search_fields_state),
             search_fields,
             searcher: None,
             input_source,
-            include_hidden: app_run_config.include_hidden,
-            errors: vec![],
-            popup: None,
-            event_sender,
-            event_receiver,
-            immediate_replace: app_run_config.immediate_replace,
-            print_results: app_run_config.print_results,
-            print_on_exit: app_run_config.print_on_exit,
-            advanced_regex: app_run_config.advanced_regex,
+            run_config: app_run_config,
+            event_channels: EventChannels::new(),
+            ui_state: UIState::new(Screen::SearchFields(search_fields_state)),
         };
 
-        if app_run_config.immediate_search || !search_field_values.search.value.is_empty() {
-            app.perform_search_if_valid();
+        if search_immediately {
+            app.perform_search_background();
         }
 
         Ok(app)
@@ -601,7 +648,11 @@ impl<'a> App {
     fn handle_app_event(&mut self, app_event: AppEvent) -> EventHandlingResult {
         match app_event {
             AppEvent::PerformSearch => {
-                self.perform_search_unwrap();
+                self.perform_search_already_validated();
+                EventHandlingResult::Rerender
+            }
+            AppEvent::DismissToast { generation } => {
+                self.dismiss_toast_if_generation_matches(generation);
                 EventHandlingResult::Rerender
             }
         }
@@ -611,7 +662,7 @@ impl<'a> App {
         if let Screen::SearchFields(SearchFieldsState {
             search_state: Some(SearchState { cancelled, .. }),
             ..
-        }) = &mut self.current_screen
+        }) = &mut self.ui_state.current_screen
         {
             cancelled.store(true, Ordering::Relaxed);
         }
@@ -619,7 +670,7 @@ impl<'a> App {
 
     fn cancel_replacement(&mut self) {
         if let Screen::PerformingReplacement(PerformingReplacementState { cancelled, .. }) =
-            &mut self.current_screen
+            &mut self.ui_state.current_screen
         {
             cancelled.store(true, Ordering::Relaxed);
         }
@@ -628,21 +679,21 @@ impl<'a> App {
     pub fn cancel_in_progress_tasks(&mut self) {
         self.cancel_search();
         self.cancel_replacement();
+
+        if let Screen::SearchFields(ref mut search_fields_state) = self.ui_state.current_screen {
+            search_fields_state.cancel_preview_updates();
+        }
     }
 
     pub fn reset(&mut self) {
         self.cancel_in_progress_tasks();
+        let mut run_config = self.run_config.clone();
+        run_config.immediate_search = false;
+
         *self = Self::new(
             self.input_source.clone(), // TODO: avoid cloning
             &SearchFieldValues::default(),
-            &AppRunConfig {
-                include_hidden: self.include_hidden,
-                advanced_regex: self.advanced_regex,
-                immediate_search: false,
-                immediate_replace: self.immediate_replace,
-                print_results: self.print_results,
-                print_on_exit: self.print_on_exit,
-            },
+            run_config,
             std::mem::take(&mut self.config),
         )
         .expect("App initialisation errors should have been detected on initial construction");
@@ -650,7 +701,7 @@ impl<'a> App {
 
     pub async fn event_recv(&mut self) -> Event {
         tokio::select! {
-            Some(event) = self.event_receiver.recv() => event,
+            Some(event) = self.event_channels.recv() => event,
             Some(bg_event) = recv_optional!(get_bg_receiver!(self)) => {
                 Event::Internal(InternalEvent::Background(bg_event))
             }
@@ -663,21 +714,80 @@ impl<'a> App {
         get_bg_receiver!(self)
     }
 
-    // Called when searching explicitly: shows error popup if validation fails
-    pub fn perform_search_if_valid(&mut self) {
+    /// Called when searching explicitly: shows error popup if there have been validation failures
+    //
+    /// NOTE: validation should have been performed (with `validate_fields`) before calling
+    // TODO: how can we enforce validation by type system?
+    fn perform_search_foreground(&mut self) {
+        if !matches!(self.ui_state.current_screen, Screen::SearchFields(_)) {
+            log::warn!(
+                "Called perform_search_with_error_popup on screen {}",
+                self.ui_state.current_screen.name()
+            );
+            return;
+        }
+
+        if !self.errors().is_empty() {
+            self.set_popup(Popup::Error);
+        } else if self.search_fields.search().text().is_empty() {
+            self.add_error(AppError {
+                name: "Search field must not be empty".to_string(),
+                long: "Please enter some search text".to_string(),
+            });
+        } else {
+            let Screen::SearchFields(ref mut search_fields_state) = self.ui_state.current_screen
+            else {
+                panic!(
+                    "Expected SearchFields, found {:?}",
+                    self.ui_state.current_screen.name()
+                );
+            };
+            search_fields_state.focussed_section = FocussedSection::SearchResults;
+            // Check if search has been performed
+            if search_fields_state.search_state.is_some() {
+                if self.run_config.immediate_replace && self.search_has_completed() {
+                    self.perform_replacement();
+                }
+            } else {
+                self.perform_search_background();
+            }
+        }
+    }
+
+    /// Called when searching in the background e.g. when entering chars into the search field: does not show
+    /// error popup if there are validation errors
+    pub fn perform_search_background(&mut self) {
+        if !matches!(self.ui_state.current_screen, Screen::SearchFields(_)) {
+            log::warn!(
+                "Called perform_search_if_valid on screen {}",
+                self.ui_state.current_screen.name()
+            );
+            return;
+        }
+
         let Some(search_config) = self.validate_fields().unwrap() else {
             return;
         };
         self.searcher = Some(search_config);
-        self.perform_search_unwrap();
+        self.perform_search_already_validated();
     }
 
     /// NOTE: validation should have been performed (with `validate_fields`) before calling
     // TODO: how can we enforce validation by type system - e.g. pass in searcher?
-    fn perform_search_unwrap(&mut self) {
-        let Screen::SearchFields(ref mut search_fields_state) = self.current_screen else {
+    fn perform_search_already_validated(&mut self) {
+        self.cancel_search();
+        let Screen::SearchFields(ref mut search_fields_state) = self.ui_state.current_screen else {
+            log::warn!(
+                "Called perform_search_unwrap on screen {}",
+                self.ui_state.current_screen.name()
+            );
             return;
         };
+        search_fields_state.cancel_preview_updates();
+        if let Some(timer) = search_fields_state.search_debounce_timer.take() {
+            timer.abort();
+        }
+
         if self.search_fields.search().text().is_empty() {
             search_fields_state.search_state = None;
         }
@@ -712,7 +822,7 @@ impl<'a> App {
         Self::spawn_search_task(
             strategy,
             background_processing_sender.clone(),
-            self.event_sender.clone(),
+            self.event_channels.sender.clone(),
             cancelled,
         );
 
@@ -728,7 +838,7 @@ impl<'a> App {
             search_state: Some(search_state),
             preview_update_state: Some(preview_update_state),
             ..
-        }) = &mut self.current_screen
+        }) = &mut self.ui_state.current_screen
         else {
             return EventHandlingResult::None;
         };
@@ -767,7 +877,7 @@ impl<'a> App {
             search_state: Some(search_state),
             preview_update_state: Some(preview_update_state),
             ..
-        }) = &mut self.current_screen
+        }) = &mut self.ui_state.current_screen
         else {
             return EventHandlingResult::None;
         };
@@ -797,7 +907,7 @@ impl<'a> App {
 
         let temp_placeholder = Screen::SearchFields(SearchFieldsState::default());
         match mem::replace(
-            &mut self.current_screen,
+            &mut self.ui_state.current_screen,
             temp_placeholder, // Will get reset if we are not on `SearchComplete` screen
         ) {
             Screen::SearchFields(SearchFieldsState {
@@ -824,7 +934,7 @@ impl<'a> App {
                             background_processing_sender.clone(),
                             cancelled.clone(),
                             replacements_completed.clone(),
-                            self.event_sender.clone(),
+                            self.event_channels.sender.clone(),
                             Some(file_searcher),
                         );
                     }
@@ -832,7 +942,8 @@ impl<'a> App {
                         let InputSource::Stdin(ref stdin) = self.input_source else {
                             panic!("Expected stdin input source, found {:?}", self.input_source)
                         };
-                        self.event_sender
+                        self.event_channels
+                            .sender
                             .send(Event::ExitAndReplace(ExitAndReplaceState {
                                 stdin: Arc::clone(stdin),
                                 replace_results: state.results,
@@ -842,7 +953,7 @@ impl<'a> App {
                     }
                 }
 
-                self.current_screen =
+                self.ui_state.current_screen =
                     Screen::PerformingReplacement(PerformingReplacementState::new(
                         background_processing_receiver,
                         cancelled,
@@ -850,7 +961,7 @@ impl<'a> App {
                         total_replacements,
                     ));
             }
-            screen => self.current_screen = screen,
+            screen => self.ui_state.current_screen = screen,
         }
     }
 
@@ -896,10 +1007,11 @@ impl<'a> App {
                     search_state: Some(state),
                     focussed_section,
                     ..
-                }) = &mut self.current_screen
+                }) = &mut self.ui_state.current_screen
                 {
                     state.set_search_completed_now();
-                    if self.immediate_replace && *focussed_section == FocussedSection::SearchResults
+                    if self.run_config.immediate_replace
+                        && *focussed_section == FocussedSection::SearchResults
                     {
                         self.perform_replacement();
                     }
@@ -907,10 +1019,10 @@ impl<'a> App {
                 EventHandlingResult::Rerender
             }
             BackgroundProcessingEvent::ReplacementCompleted(replace_state) => {
-                if self.print_results {
+                if self.run_config.print_results {
                     EventHandlingResult::new_exit_stats(replace_state)
                 } else {
-                    self.current_screen = Screen::Results(replace_state);
+                    self.ui_state.current_screen = Screen::Results(replace_state);
                     EventHandlingResult::Rerender
                 }
             }
@@ -933,7 +1045,7 @@ impl<'a> App {
         if let Screen::SearchFields(SearchFieldsState {
             search_state: Some(search_in_progress_state),
             ..
-        }) = &mut self.current_screen
+        }) = &mut self.ui_state.current_screen
         {
             let mut results_with_replacements = Vec::new();
             let searcher = self
@@ -975,34 +1087,7 @@ impl<'a> App {
                 EventHandlingResult::Rerender
             }
             CommandSearchFocusFields::TriggerSearch => {
-                if !self.errors().is_empty() {
-                    self.set_popup(Popup::Error);
-                } else if self.search_fields.search().text().is_empty() {
-                    self.add_error(AppError {
-                        name: "Search field must not be empty".to_string(),
-                        long: "Please enter some search text".to_string(),
-                    });
-                } else {
-                    let Screen::SearchFields(ref mut search_fields_state) = self.current_screen
-                    else {
-                        panic!(
-                            "Expected SearchFields, found {:?}",
-                            self.current_screen.name()
-                        );
-                    };
-                    search_fields_state.focussed_section = FocussedSection::SearchResults;
-                    // Check if search has been performed
-                    if search_fields_state.search_state.is_some() {
-                        if self.immediate_replace && self.search_has_completed() {
-                            self.perform_replacement();
-                        }
-                    } else {
-                        if let Some(timer) = search_fields_state.search_debounce_timer.take() {
-                            timer.abort();
-                        }
-                        self.perform_search_if_valid();
-                    }
-                }
+                self.perform_search_foreground();
                 EventHandlingResult::Rerender
             }
             CommandSearchFocusFields::FocusPreviousField => {
@@ -1026,7 +1111,7 @@ impl<'a> App {
         key_code: KeyCode,
         key_modifiers: KeyModifiers,
     ) -> EventHandlingResult {
-        let Screen::SearchFields(ref mut search_fields_state) = self.current_screen else {
+        let Screen::SearchFields(ref mut search_fields_state) = self.ui_state.current_screen else {
             return EventHandlingResult::None;
         };
         if let FieldName::FixedStrings = self.search_fields.highlighted_field().name {
@@ -1046,7 +1131,7 @@ impl<'a> App {
         } else {
             return EventHandlingResult::Rerender;
         }
-        let Screen::SearchFields(ref mut search_fields_state) = self.current_screen else {
+        let Screen::SearchFields(ref mut search_fields_state) = self.ui_state.current_screen else {
             return EventHandlingResult::None;
         };
         let file_searcher = self
@@ -1086,7 +1171,7 @@ impl<'a> App {
             if let Some(timer) = search_fields_state.search_debounce_timer.take() {
                 timer.abort();
             }
-            let event_sender = self.event_sender.clone();
+            let event_sender = self.event_channels.sender.clone();
             search_fields_state.search_debounce_timer = Some(tokio::spawn(async move {
                 tokio::time::sleep(Duration::from_millis(300)).await;
                 let _ =
@@ -1097,7 +1182,8 @@ impl<'a> App {
     }
 
     fn get_search_state_unwrap(&mut self) -> &mut SearchState {
-        self.current_screen
+        self.ui_state
+            .current_screen
             .unwrap_search_fields_state_mut()
             .search_state
             .as_mut()
@@ -1111,9 +1197,9 @@ impl<'a> App {
         event: CommandSearchFocusResults,
     ) -> EventHandlingResult {
         assert!(
-            matches!(self.current_screen, Screen::SearchFields(_)),
+            matches!(self.ui_state.current_screen, Screen::SearchFields(_)),
             "Expected current_screen to be SearchFields, found {}",
-            self.current_screen.name()
+            self.ui_state.current_screen.name()
         );
 
         match event {
@@ -1123,18 +1209,25 @@ impl<'a> App {
             }
             CommandSearchFocusResults::BackToFields => {
                 self.cancel_search();
-                let search_fields_state = self.current_screen.unwrap_search_fields_state_mut();
+                let search_fields_state = self
+                    .ui_state
+                    .current_screen
+                    .unwrap_search_fields_state_mut();
                 search_fields_state.focussed_section = FocussedSection::SearchFields;
                 EventHandlingResult::Rerender
             }
             CommandSearchFocusResults::OpenInEditor => {
-                let search_fields_state = self.current_screen.unwrap_search_fields_state_mut();
+                let search_fields_state = self
+                    .ui_state
+                    .current_screen
+                    .unwrap_search_fields_state_mut();
                 if let Some(ref mut search_in_progress_state) = search_fields_state.search_state {
                     let selected = search_in_progress_state
                         .primary_selected_field_mut()
                         .expect("Expected to find selected field");
                     if let Some(ref path) = selected.search_result.path {
-                        self.event_sender
+                        self.event_channels
+                            .sender
                             .send(Event::LaunchEditor((
                                 path.clone(),
                                 selected.search_result.line_number,
@@ -1222,7 +1315,7 @@ impl<'a> App {
             }
         }
 
-        match &mut self.current_screen {
+        match &mut self.ui_state.current_screen {
             Screen::SearchFields(search_fields_state) => {
                 let Command::SearchFields(command) = command else {
                     panic!("Expected SearchFields command, found {command:?}");
@@ -1231,6 +1324,16 @@ impl<'a> App {
                 match command {
                     CommandSearchFields::TogglePreviewWrapping => {
                         self.config.preview.wrap_text = !self.config.preview.wrap_text;
+                        self.show_toggle_toast("Text wrapping", self.config.preview.wrap_text);
+                        EventHandlingResult::Rerender
+                    }
+                    CommandSearchFields::ToggleHiddenFiles => {
+                        if matches!(self.input_source, InputSource::Stdin(_)) {
+                            return EventHandlingResult::None;
+                        }
+                        self.run_config.include_hidden = !self.run_config.include_hidden;
+                        self.show_toggle_toast("Hidden files", self.run_config.include_hidden);
+                        self.perform_search_background();
                         EventHandlingResult::Rerender
                     }
                     CommandSearchFields::SearchFocusFields(command) => {
@@ -1273,11 +1376,13 @@ impl<'a> App {
         &mut self,
         key_event: KeyEvent,
     ) -> Either<Command, EventHandlingResult> {
-        let maybe_event = self.key_map.lookup(&self.current_screen, key_event);
+        let maybe_event = self
+            .key_map
+            .lookup(&self.ui_state.current_screen, key_event);
 
         // Quit should take precedent over closing popup etc.
         if !matches!(maybe_event, Some(Command::General(CommandGeneral::Quit))) {
-            if self.popup.is_some() {
+            if self.ui_state.popup.is_some() {
                 self.clear_popup();
                 return Right(EventHandlingResult::Rerender);
             }
@@ -1300,7 +1405,7 @@ impl<'a> App {
             }
 
             // If we're in SearchFields focus, treat unmatched keys as text input
-            if let Screen::SearchFields(state) = &self.current_screen {
+            if let Screen::SearchFields(state) = &self.ui_state.current_screen {
                 if state.focussed_section == FocussedSection::SearchFields {
                     Command::SearchFields(CommandSearchFields::SearchFocusFields(
                         CommandSearchFocusFields::EnterChars(key_event.code, key_event.modifiers),
@@ -1320,7 +1425,7 @@ impl<'a> App {
             search_text: self.search_fields.search().text(),
             replacement_text: self.search_fields.replace().text(),
             fixed_strings: self.search_fields.fixed_strings().checked,
-            advanced_regex: self.advanced_regex,
+            advanced_regex: self.run_config.advanced_regex,
             match_whole_word: self.search_fields.whole_word().checked,
             match_case: self.search_fields.match_case().checked,
         };
@@ -1328,7 +1433,8 @@ impl<'a> App {
             InputSource::Directory(directory) => Some(DirConfig {
                 include_globs: Some(self.search_fields.include_files().text()),
                 exclude_globs: Some(self.search_fields.exclude_files().text()),
-                include_hidden: self.include_hidden,
+                include_hidden: self.run_config.include_hidden,
+                include_git_folders: self.run_config.include_git_folders,
                 directory: directory.clone(),
             }),
             InputSource::Stdin(_) => None,
@@ -1435,31 +1541,66 @@ impl<'a> App {
     }
 
     pub fn show_popup(&self) -> bool {
-        self.popup.is_some()
+        self.ui_state.popup.is_some()
     }
 
     pub fn popup(&self) -> Option<&Popup> {
-        self.popup.as_ref()
+        self.ui_state.popup.as_ref()
     }
 
     pub fn errors(&self) -> Vec<AppError> {
-        let app_errors = self.errors.clone().into_iter();
+        let app_errors = self.ui_state.errors().iter().cloned();
         let field_errors = self.search_fields.errors().into_iter();
         app_errors.chain(field_errors).collect()
     }
 
     pub fn add_error(&mut self, error: AppError) {
-        self.popup = Some(Popup::Error);
-        self.errors.push(error);
+        self.ui_state.popup = Some(Popup::Error);
+        self.ui_state.add_error(error);
     }
 
     fn clear_popup(&mut self) {
-        self.popup = None;
-        self.errors.clear();
+        self.ui_state.popup = None;
+        self.ui_state.clear_errors();
     }
 
     fn set_popup(&mut self, popup: Popup) {
-        self.popup = Some(popup);
+        self.ui_state.popup = Some(popup);
+    }
+
+    pub fn toast_message(&self) -> Option<&str> {
+        self.ui_state.toast.as_ref().map(|t| t.message.as_str())
+    }
+
+    fn show_toast(&mut self, message: String) {
+        let generation = self.ui_state.toast.as_ref().map_or(1, |t| t.generation + 1);
+        self.ui_state.toast = Some(Toast {
+            message,
+            generation,
+        });
+
+        let toast_timeout_ms = 1500;
+
+        let event_sender = self.event_channels.sender.clone();
+        tokio::spawn(async move {
+            tokio::time::sleep(Duration::from_millis(toast_timeout_ms)).await;
+            let _ = event_sender.send(Event::Internal(InternalEvent::App(
+                AppEvent::DismissToast { generation },
+            )));
+        });
+    }
+
+    fn show_toggle_toast(&mut self, name: &str, enabled: bool) {
+        let status = if enabled { "ON" } else { "OFF" };
+        self.show_toast(format!("{name}: {status}"));
+    }
+
+    fn dismiss_toast_if_generation_matches(&mut self, generation: u64) {
+        if let Some(toast) = &self.ui_state.toast
+            && toast.generation == generation
+        {
+            self.ui_state.toast = None;
+        }
     }
 
     pub fn keymaps_all(&self) -> Vec<(String, String)> {
@@ -1490,7 +1631,7 @@ impl<'a> App {
             };
         }
 
-        let current_screen_keys = match &self.current_screen {
+        let current_screen_keys = match &self.ui_state.current_screen {
             Screen::SearchFields(search_fields_state) => {
                 let mut keys = vec![];
                 match search_fields_state.focussed_section {
@@ -1588,6 +1729,13 @@ impl<'a> App {
                     "toggle text wrapping in preview",
                     Show::FullOnly,
                 ));
+                if matches!(self.input_source, InputSource::Directory(_)) {
+                    keys.push(keymap!(
+                        search.toggle_hidden_files,
+                        "toggle hidden files",
+                        Show::FullOnly,
+                    ));
+                }
                 keys
             }
             Screen::PerformingReplacement(_) => vec![],
@@ -1603,7 +1751,7 @@ impl<'a> App {
             }
         };
 
-        let on_search_results = if let Screen::SearchFields(ref s) = self.current_screen {
+        let on_search_results = if let Screen::SearchFields(ref s) = self.ui_state.current_screen {
             s.focussed_section == FocussedSection::SearchResults
         } else {
             false
@@ -1652,7 +1800,7 @@ impl<'a> App {
     }
 
     fn multiselect_enabled(&self) -> bool {
-        match &self.current_screen {
+        match &self.ui_state.current_screen {
             Screen::SearchFields(SearchFieldsState {
                 search_state: Some(state),
                 ..
@@ -1662,14 +1810,14 @@ impl<'a> App {
     }
 
     fn toggle_multiselect_mode(&mut self) {
-        match &mut self.current_screen {
+        match &mut self.ui_state.current_screen {
             Screen::SearchFields(SearchFieldsState {
                 search_state: Some(state),
                 ..
             }) => state.toggle_multiselect_mode(),
             _ => panic!(
                 "Tried to disable multi-select on {:?}",
-                self.current_screen.name()
+                self.ui_state.current_screen.name()
             ),
         }
     }
@@ -1685,7 +1833,7 @@ impl<'a> App {
             search_state: Some(state),
             search_debounce_timer,
             ..
-        }) = &self.current_screen
+        }) = &self.ui_state.current_screen
         {
             state.search_completed.is_some()
                 && search_debounce_timer
@@ -1705,7 +1853,7 @@ impl<'a> App {
                 }),
             preview_update_state,
             ..
-        }) = &self.current_screen
+        }) = &self.ui_state.current_screen
         {
             processing_receiver.is_empty()
                 && preview_update_state
@@ -1779,7 +1927,7 @@ impl ValidationErrorHandler for AppErrorHandler {
 
 #[cfg(test)]
 mod tests {
-    use frep_core::{
+    use crate::{
         line_reader::LineEnding,
         replace::{ReplaceResult, ReplaceStats},
         search::{SearchResult, SearchResultWithReplacement},
@@ -1963,7 +2111,7 @@ mod tests {
 
         let (results, _num_ignored) =
             crate::replace::split_results(search_results_with_replacements);
-        let stats = frep_core::replace::calculate_statistics(results);
+        let stats = crate::replace::calculate_statistics(results);
 
         assert_eq!(
             stats,
@@ -1987,7 +2135,7 @@ mod tests {
 
         let (results, _num_ignored) =
             crate::replace::split_results(search_results_with_replacements);
-        let stats = frep_core::replace::calculate_statistics(results);
+        let stats = crate::replace::calculate_statistics(results);
 
         assert_eq!(
             stats,
@@ -2294,7 +2442,7 @@ mod tests {
         let mut app = App::new(
             InputSource::Directory(std::env::current_dir().unwrap()),
             &SearchFieldValues::default(),
-            &AppRunConfig::default(),
+            AppRunConfig::default(),
             Config::default(),
         )
         .unwrap();
@@ -2311,7 +2459,7 @@ mod tests {
         let mut app = App::new(
             InputSource::Directory(std::env::current_dir().unwrap()),
             &SearchFieldValues::default(),
-            &AppRunConfig::default(),
+            AppRunConfig::default(),
             Config::default(),
         )
         .unwrap();
