@@ -5,6 +5,9 @@ pub mod diff;
 pub mod errors;
 pub mod fields;
 pub mod keyboard;
+pub mod line_reader;
 pub mod replace;
+pub mod run;
 pub mod search;
 pub mod utils;
+pub mod validation;
