@@ -1,7 +1,7 @@
 use anyhow::bail;
 use clap::Parser;
-use frep_core::validation::{DirConfig, SearchConfig};
 use log::LevelFilter;
+use scooter_core::validation::{DirConfig, SearchConfig};
 use std::{
     io::{self, IsTerminal, Read},
     path::PathBuf,
@@ -37,6 +37,10 @@ struct Args {
     #[arg(short = '.', long, action = clap::ArgAction::SetTrue)]
     hidden: bool,
 
+    /// Include `.git` folders (both at the root and in subdirectories)
+    #[arg(long, action = clap::ArgAction::SetTrue)]
+    include_git_folders: bool,
+
     /// Log level (trace, debug, info, warn, error)
     #[arg(
         long,
@@ -81,6 +85,10 @@ struct Args {
     #[arg(long)]
     print_on_exit: bool,
 
+    /// Override the editor command for opening files (overrides config file setting). Use %file and %line as placeholders.
+    #[arg(long)]
+    editor_command: Option<String>,
+
     // --- Initial values for fields ---
     //
     /// Text to search with
@@ -110,10 +118,6 @@ struct Args {
     /// Glob patterns, separated by commas (,), that file paths must not match
     #[arg(short = 'E', long)]
     files_to_exclude: Option<String>,
-
-    /// Override the editor command for opening files (overrides config file setting). Use %file and %line as placeholders.
-    #[arg(long)]
-    editor_command: Option<String>,
 }
 
 fn parse_log_level(s: &str) -> Result<LevelFilter, String> {
@@ -188,6 +192,9 @@ fn validate_stdin_usage(args: &Args, stdin_content: Option<&str>) -> anyhow::Res
         if args.hidden {
             bail!("Cannot use --hidden flag when processing stdin");
         }
+        if args.include_git_folders {
+            bail!("Cannot use --include-git-folders flag when processing stdin");
+        }
         if args.files_to_include.is_some() {
             bail!("Cannot use --files-to-include when processing stdin");
         }
@@ -218,6 +225,7 @@ impl<'a> TryFrom<&'a Args> for AppConfig<'a> {
             search_field_values: args.into(),
             app_run_config: AppRunConfig {
                 include_hidden: args.hidden,
+                include_git_folders: args.include_git_folders,
                 advanced_regex: args.advanced_regex,
                 immediate_search: args.immediate_search || immediate,
                 immediate_replace: args.immediate_replace || immediate,
@@ -293,6 +301,7 @@ fn dir_config_from_args(args: &Args) -> DirConfig<'_> {
         include_globs: args.files_to_include.as_deref(),
         exclude_globs: args.files_to_exclude.as_deref(),
         include_hidden: args.hidden,
+        include_git_folders: args.include_git_folders,
         directory: args.directory.clone(),
     }
 }
@@ -319,6 +328,7 @@ mod tests {
         Args {
             directory: env::current_dir().unwrap(),
             hidden: false,
+            include_git_folders: false,
             log_level: LevelFilter::Info,
             advanced_regex: false,
             immediate_search: false,
